from sqlalchemy import exc

from database import db
import logging

# We "reset" the alembic version history for Stackspin 2.2, to clean up our old
# mess of database migrations a bit, and in particular to make the transition
# easier to moving the source of truth for some of the data (list of apps) out
# of the database and into configmaps. This function deals with older clusters
# that have to be led through this transition. To determine if we need to do
# anything, we look at the `alembic_version` value in the database. If it's a
# legacy version, we delete the table so the alembic migration will view the
# database as "empty" and perform all new migrations on it. The new initial
# migration will have to handle that case specially, by checking if any tables
# already exist, and not do anything in that case.
def reset():
    logging.info("Checking if alembic version needs to be reset.")
    version = None
    try:
        result = db.session.execute("select version_num from alembic_version")
        for row in result:
            version = row[0]
    except exc.ProgrammingError:
        # We assume this means the alembic_version table doesn't exist, which
        # is expected for new clusters.
        pass
    logging.info(f"alembic version: {version}")
    legacy_versions = ["fc0892d07771", "3fa0c38ea1ac", "e08df0bef76f", "b514cca2d47b", "5f462d2d9d25", "27761560bbcb"]
    if version in legacy_versions:
        logging.info("This is an old version: resetting.")
        db.session.execute("drop table alembic_version")
    else:
        logging.info("This is not a known legacy version: not resetting.")
        
